from entity import Component

# contains a list of all the public, therefore removable behaviors
class Configurable(Component):
	def __init__(self):
		self.behaviors = []
		self.active = False
		self.window = None

# is a component container that can attach a type to another entity
class Behavior(Component):
	def __init__(self, esys, component_t, args):
		self.esys = esys
		self.component_t = component_t
		self.attached = None
		self.args = args

	def set_entity(self, entity):
		self.my_entity = entity
	
	def attach(self, entity):
		# ensure good usage of this
		# shouldn't be able to attach behaviors to behaviors
		assert(self.attached == None)
		m = self.esys
		assert(not m.has_component(entity, self.component_t))
		assert(m.has_component(entity, Configurable))

		# add self to list
		config = m.get_component(entity, Configurable)
		config.behaviors.append(self.my_entity)

		# add
		if self.args == None:
			m.add_component(entity, self.component_t)
		else:
			m.add_component(entity, self.component_t(*self.args))
		self.attached = entity
	
	def remove(self):
		target = self.attached
		my_type = self.component_t

		assert(target != None)
		m = self.esys
		assert(m.has_component(target, my_type))

		# remove self from Configurable
		config = m.get_component(target, Configurable)
		config.behaviors.remove(self.my_entity)

		# remove component from Configurable
		m.remove_component(target, my_type)

		assert(not m.has_component(target, my_type))

		self.attached = None

