from entity import Component
from geom import vector, rectangle

from view_components import *

class BlocksLevel(Component):
	def __repr__(self): return 'BlocksLevel'

# model components
class Position(Component, vector):
	def __init__(self, x, y):
		vector.__init__(self, x, y)
	def __str__(self):
		return 'Position: {0}'.format(vector.__repr__(self))

class Size(Component, vector):
	def __init__(self, w, h):
		vector.__init__(self, w, h)
	def __str__(self):
		return 'Size: {0}'.format(vector.__repr__(self))
	
class Velocity(Component, vector):
	def __init__(self, dx, dy):
		vector.__init__(self, dx, dy)
	def __str__(self):
		return 'Velocity: {0}'.format(vector.__repr__(self))

class SimpleValue(Component):
	def __init__(self, val):
		self.value = val

class Friction(SimpleValue): pass

class SurvivalRequired(Component):
	def __repr__(self): return 'SurvivalRequired'
class WrapToWorld(Component): 
	def __repr__(self): return 'WrapToWorld'
class Gravity(Component):
	def __repr__(self): return 'Gravity'

class Obstacle(Component):
	def __init__(self, top=True, left=True, right=True, bottom=True):
		self.top = True
		self.left = True
		self.right = True
		self.bottom = True

class ConstantMovement(Component):
	def __init__(self, velocity):
		self.velocity = velocity

class BounceMovement(Component):
	pass

# GUI Components:
class Response(Component):
	def __init__(self, callback, times=-1):
		self.callback = callback
		self.times = times
	
	def fire(self, *args):
		self.times-=1
		self.callback(*args)
		return self.is_done()
	
	def is_done(self):
		return self.times == 0

class ClickResponse(Response):
	def __repr__(self):
		return "ClickResponse"

class RegionResponse(Response):
	def _repr__(self):
		return "RegionResponse"

class Draggable(Component):
	def __init__(self, bounds):
		self.bounds = bounds
		self.held = False
		self.anchor_x = 0
		self.anchor_y = 0
	def start(self):
		assert(not self.held)
		self.held = True
	def end(self):
		assert(self.held)
		self.held = False
	def __repr__(self):
		return "Draggable"
	def is_held(self):
		return self.held

class Tooltip(Component):
	def __init__(self, text):
		self.text = text
		self.mouse_over = False
		self.entity = None

	def activate(self, esys, tooltip):
		self.entity = tooltip
		self.esys = esys
		self.mouse_over = True
	
	def deactivate(self, esys):
		if(self.mouse_over == False):
			return
		else:
			self.mouse_over = False
			esys.kill_entity(self.entity)
			self.entity = None

class Parent(Component):
	def __del__(self):
		#print 'Parent.__del__(parent:{0}, me:{1})'.format(self.parent, self.my_entity)
		m = self.esys
		if m.has_component(self.parent, Children):
			m.get_component(self.parent, Children).remove(self.my_entity)

	def __init__(self, esys, my_entity, parent_entity):
		#print 'Parent({0})'.format(my_entity)
		self.esys = esys
		self.my_entity = my_entity
		self.parent = parent_entity

class Children(Component):
	def __init__(self, esys, my_entity, children=[]):
		self.esys = esys
		self.my_entity = my_entity
		self.entities = []
		for c in children:
			#print c
			self.add(c)
	
	def add(self, child):
		self.entities.append(child)
		self.esys.add_component(child, Parent(self.esys, child, self.my_entity))

	def remove(self, child):
		#print 'remove {0}'.format(child)
		self.entities.remove(child)

	def __del__(self):
		#print 'Children.__del__'
		for e in self.entities:
			#print e
			self.esys.kill_entity(e)

# contains a pointer to an entity that has a behavior
class Selectable(Component):
	def __init__(self, logical_entity):
		self.entity = logical_entity
		self.selected = False


