from geom import vector

class Component(object):
	pass

class EntitySystem(object):
	# map components "classes" to map of entity id to data
	def __init__(self):
		self.data = {}
		self.index = 0

	def get_all_entities(self):
		entities = set() 
		for entity_dict in self.data.values():
			entities = entities.union(entity_dict.keys())
		return entities

	def get_all_components(self, e):
		assert(isinstance(e, int))
		components = []
		for component_t in self.data:
			if self.has_component(e, component_t):
				components.append(self.get_component(e, component_t))
		return components

	def __str__(self):
		out = ''
		for e in self.get_all_entities():
			out += '  E[{0}]=[\n'.format(e)
			components = self.get_all_components(e)
			for c in components:
				out += '    {0}\n'.format(c)
			out += '  ]\n'
		return out

	def make_entity(self, *components):
		e = self.new_entity()
		for c in components:
			self.add_component(e,c)
		return e
	
	def new_entity(self):
		e = self.index
		self.index += 1
		return e

	def kill_entity(self, e):
		assert(isinstance(e, int))
		for component_t in self.data:
			entities = self.data[component_t]
			if e in entities:
				del entities[e]
	
	def add_component(self, e, component):
		assert(isinstance(e, int))
		
		# accept either a class or an instance
		if isinstance(component, type):
			component_t = component
			component = component_t()
		else:
			component_t = component.__class__
		
		assert(isinstance(component, component_t))

		if component_t not in self.data:
			self.data[component_t] = {}

		elist = self.data[component_t]
		elist[e] = component
	
	def get_entities(self, component_t):
		if component_t not in self.data:
			return {}
		else:
			return self.data[component_t]
	
	def has_component(self, e, component_t):
		assert(isinstance(e, int))
		if(component_t not in self.data):
			return False

		return e in self.data[component_t]

	def remove_component(self, e, component_t):
		assert(isinstance(e, int))
		elist = self.data[component_t]
		del elist[e]

	def get_component(self, e, component_t):
		assert(isinstance(e, int))
		elist = self.data[component_t]
		return elist[e]

	def set_component(self, e, component_t, val):
		assert(isinstance(e, int))
		elist = self.data[component_t]
		elist[e] = val

