from components import *
from behaviors import *
from view import *
from copy import deepcopy

# gui components:

def make_confirm(esys, text, callback, times=1):
	return esys.make_entity(
			Renderable(1000),
			BlocksLevel(),
			Position(0,0),
			Size(Screen.Width, Screen.Height),
			FontColor(255,255,255),
			FillColor(0,0,0,200),
			LineStyle(Color(255,255,255), 2),
			DisplayQuad(),
			DisplayBorder(),
			ClickResponse(callback, times),
			DisplayText(text)
			)

def make_tooltip(esys, parent, text):
	popup = esys.make_entity(
			Renderable(900),
			Position(Screen.Width/2 - 200, 8),
			Size(400,32),
			FontColor(255,255,255),
			FillColor(0,0,0,200),
			LineStyle(Color(255,255,255), 2),
			DisplayQuad(),
			DisplayBorder(),
			DisplayText(text)
			)
	esys.add_component(parent, Children(esys, parent, [popup]))

	return popup

def make_customize_window(esys, subject):
	m = esys
	assert(m.has_component(subject, Configurable))

	m.get_component(subject, Selectable).selected = True
	
	window_pos = Position(Screen.Width/2 - 120, 48)
	window_size = Size(240, 240)
	
	window = esys.make_entity(
			Renderable(800),
			window_pos,
			window_size,
			FillColor(0,0,0,200),
			LineStyle(Color(0,0,0), 2),
			DisplayQuad(),
			DisplayBorder(),
			Selectable(subject),
			Draggable(Screen.rect()),
			BlocksLevel()
			)

	m.add_component(window, DisplayConnection(subject, window))

	config = m.get_component(subject, Configurable)
	config.active = True
	config.window = window
	
	children = Children(esys, window, [])

	n = 0
	cols = int(window_size.x // 32)
	for i in config.behaviors:
		x = n % cols
		y = int(n // cols)

		behavior = m.get_component(i, Behavior)

		children.add(esys.make_entity(
			Renderable(810),
			Position(window_pos.x + x * 48, window_pos.y + window_size.y - (y+2)*48),
			Size(32,32),
			FillColor(0,255,255),
			LineStyle(Color(0,0,0), 1),
			Tooltip(m.get_component(i, Tooltip).text), # copy tooltip from behavior
			#Tooltip("This is a tip"),
			ClickResponse(lambda : behavior.remove(), 1),
			#Selectable(i),
			DisplayQuad(),
			DisplayBorder()
			))

		# increase count!
		n+=1
	
	def _on_close_window(esys, window, subject):
		esys.get_component(subject, Configurable).active = False
		esys.kill_entity(window)

	children.add(make_button(
		m,
		"X",
		window_pos.x + window_size.x - 40,
		window_pos.y + window_size.y - 40,
		32,
		lambda : _on_close_window(m, window, subject)
		))

	# add children to window entity so they get cleaned up when it disappears
	m.add_component(window, children)

def make_button(esys, text, x, y, w, callback):
	return esys.make_entity(
			Renderable(820),
			Position(x, y),
			Size(w,32),
			FontColor(0,0,0),
			FillColor(200,200,200),
			LineStyle(Color(255,255,255), 2),
			DisplayQuad(),
			DisplayBorder(),
			ClickResponse(callback, 1),
			DisplayText(text)
			)

def make_textbox(lvl, text, x, y, w):
	return lvl.make_entity(
			Renderable(100),
			Position(x, y),
			Size(w,40),
			FontColor(0,0,0),
			FillColor(200,200,200),
			LineStyle(Color(100,100,100), 1),
			DisplayText(text),
			DisplayQuad(),
			DisplayBorder(),
			Obstacle()
			)


def make_sheep(lvl, x, y):
	sheep = lvl.make_entity(
			Renderable(200),
			Position(x,y),
			Velocity(0,0),
			Size(32,32),
			FillColor(255,0,0),
			DisplayQuad(),
			SurvivalRequired(),
			Configurable(),
			WrapToWorld(),
			#Obstacle(),
			#Draggable(Screen.rect()),
			Gravity()
			)

	lvl.esys.add_component(sheep, Selectable(sheep))
	#test = make_behavior(lvl, "Charlie", 56, 56, Velocity, (4,5))
	#lvl.esys.get_component(test, Behavior).attach(sheep)

	return sheep

def make_goal(lvl, x, y, w, h, text="Goal"):
	goal = lvl.make_entity(
			Renderable(100),
			Position(x,y),
			Size(w,h),
			FillColor(0xff,0xb9,0x0f,200),
			FontColor(0x8b,0x65,0x08),
			LineStyle(Color(0x8b,0x65,0x08), 1),
			DisplayBorder(),
			DisplayQuad(),
			RegionResponse(lvl.on_goal),
			DisplayText(text)
			)

def make_platform(lvl, x, y, w, h):
	return lvl.make_entity(
			Renderable(100),
			Position(x,y),
			#Velocity(20.0,0),
			Size(w,h),
			FillColor(60,60,60),
			Obstacle(),
			Friction(0.9),
			DisplayQuad()
			)

def make_behavior(lvl, text, x, y, component_t, args):
	e = lvl.make_entity(
			Renderable(200),
			Position(x,y),
			Size(32,32),
			FillColor(0,255,255),
			LineStyle(Color(0,0,0), 1),
			Tooltip(text),
			Behavior(lvl.esys, component_t, args),
			DisplayQuad(),
			DisplayBorder()
			)

	lvl.esys.add_component(e, Selectable(e))
	lvl.esys.get_component(e, Behavior).set_entity(e)
	return e


