from components import *
from geom import rectangle
from physics import *
from factory import *

class GUISystem(object):
	def __init__(self, esys):
		self.esys = esys

	def on_mouse_release(self, x, y, buttons, modifiers):
		#print 'on_mouse_release{0}'.format((x,y,buttons,modifiers))
		self.update_drag_position(x,y)
		self.finish_drag()
		
	def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
		#print 'on_mouse_drag{0}'.format((x,y,dx,dy, buttons, modifiers))
		self.update_drag_position(x,y)

	def on_mouse_press(self, x, y, button, modifiers):
		#print 'on_mouse_press{0}'.format((x,y,button,modifiers))
		#if button != Mouse.LEFT:
			#return
		self.check_click(x, y)
		self.check_select(x,y)
		self.check_customize(x,y)
		self.check_start_drag(x,y)

	def on_mouse_motion(self, x, y, dx, dy):
		#print (x,y,dx,dy)

		m = self.esys
		item = self.entity_under_xy(Tooltip, x, y)
		
		if(item != None):
			over = m.get_component(item, Tooltip)
			if(not over.mouse_over):
				over.activate(m, make_tooltip(m, item, over.text))

		for e in m.get_entities(Tooltip):
			cur = m.get_component(e, Tooltip)
			if(e != item and cur.mouse_over):
				cur.deactivate(m)
	
	def update_drag_position(self,x,y):
		(e, drag) = self.get_entity_held_by_mouse()
		if(e == None):
			return

		m = self.esys
		pos = m.get_component(e, Position)
		drag = m.get_component(e, Draggable)
		next_pos = Position(x - drag.anchor_x, y - drag.anchor_y)
		
		dx = next_pos.x - pos.x
		dy = next_pos.y - pos.y

		# move children, too
		if m.has_component(e, Children):
			children = m.get_component(e, Children)
			for c in children.entities:
				m.get_component(c, Position).move_relative(dx, dy)

		m.set_component(e, Position, next_pos)

	def get_entity_held_by_mouse(self):
		m = self.esys
		for e in m.get_entities(Draggable):
			drag = m.get_component(e, Draggable)
			if drag.is_held():
				return (e, drag)
		return (None, None)

	def finish_drag(self):
		(e, drag) = self.get_entity_held_by_mouse()
		if(e == None):
			return
		drag.end()

	def check_start_drag(self, x, y):
		(e, drag) = self.get_entity_held_by_mouse()
		if e != None:
			return

		e = self.entity_under_xy(Draggable, x, y)
		if e == None:
			return

		m = self.esys
		pos = m.get_component(e, Position)
		drag = m.get_component(e, Draggable)
		
		drag.anchor_x = x - pos.x
		drag.anchor_y = y - pos.y
		drag.start()

	def check_customize(self, x, y):
		e = self.entity_under_xy(Configurable, x, y)

		if e == None:
			return
		
		m = self.esys
		config = m.get_component(e, Configurable)
		if config.active: # don't open duplicate windows
			return

		make_customize_window(self.esys, e)

	def check_select(self, x, y):
		m = self.esys
		e = self.entity_under_xy(Selectable, x, y)

		# selected nothing : equivalent of remove behavior if behavior selected
		if e == None:
		#	print "Selected nothing"
		#	for s in m.get_entities(Selectable):
		#		c = m.get_component(s, Selectable)

		#		if c.selected:
		#			target = c.entity
		#			if m.has_component(target, Behavior):
		#				m.get_component(target, Behavior).remove()
		#			# regardless, no longer selected
		#			m.get_component(s, Selectable).selected = False
			return

		# e is valid

		# clicked on a behavior
		if m.has_component(e, Behavior):
			result = self.get_selected(Configurable)
			
			l = len(result)
			assert( l == 0 or l == 1) # only one at a time
			if l: # there is one
				dest = result[0]
				self.add_behavior(e, dest)
				self.close_all_windows()
				m.get_component(dest, Configurable).selected = False
			else:
				print "Selected behavior {0}".format(e)
				self.get_selected(Behavior, True) # find and deselect others
				m.get_component(e, Selectable).selected = True # select this
			
			return
		
		# clicked on a behavior drop
		elif m.has_component(e, Configurable):
			print "Selected configurable {0}".format(e)

			# find and deselect all currently selected components
			# if there are any, we just select this and are done,
			# there won't be any behaviors by convention
			if len(self.get_selected(Configurable, True)):
				# toggle selection truth value
				return 
			
			# add currently selected behaviors to e
			selected_behaviors = self.get_selected(Behavior, True) 
			
			if len(selected_behaviors):
				for b_id in selected_behaviors:
					b = m.get_component(b_id, Behavior)
					self.add_behavior(b_id, e)
				
				config = m.get_component(e, Configurable)
				
				self.close_all_windows()

	# remove behavior of type if one already exists, replace with this
	def add_behavior(self, src, dest):
		m = self.esys
		
		to_add = m.get_component(src, Behavior)
		dest_config = m.get_component(dest, Configurable)

		for already in dest_config.behaviors:
			if m.has_component(already, to_add.component_t):
				print 'already has'
				to_remove = m.get_component(already, Behavior)
				to_remove.remove()

		to_add.attach(dest)
	
	def close_all_windows(self):
		m = self.esys

		for e in m.get_entities(Configurable):
			config = m.get_component(e, Configurable)
			if config.active:
				m.kill_entity(config.window)
				config.active = False
	
	# gets all selected with component of Type
	def get_selected(self, Type, deselect=False):
		m = self.esys
		result = []
		for e in m.get_entities(Selectable):
			if m.has_component(e, Type):
				s = m.get_component(e, Selectable)
				if s.selected:
					result.append(e)
					m.get_component(e, Selectable).selected = (not deselect)
		return result
			
			



	def check_click(self, x, y):
		e = self.entity_under_xy(ClickResponse, x, y)
		if e == None:
			return

		m = self.esys
		resp = m.get_component(e, ClickResponse)
		# will return false if it should happen more times
		if resp.fire():
			m.kill_entity(e)

	def entity_under_xy(self, Type, x, y):
		m = self.esys
		for e in m.get_entities(Type):
			
			# skip attached behavior
			if m.has_component(e, Behavior):
				behavior = m.get_component(e, Behavior)
				if behavior.attached != None:
					return None


			pos = m.get_component(e, Position)
			size = m.get_component(e, Size)
			
			hit_rect = rectangle(pos.x, pos.y, size.x, size.y)

			if hit_rect.contains_xy(x,y):
				return e
		return None

