from components import *
from behaviors import *
from geom import *
from factory import *
from view import Screen
from physics import *


class Level(object):
	def __init__(self, esys, index=0):
		self.esys = esys
		self.index = index
		self.entities = []
		self.goals = 0
		self.goals_required = 999

		self.load_level()

	def load_level(self):
		# remove any old components
		self.clear()

		n = self.index

		if(n == 0):
			self.goals_required = 1
			
			make_textbox(self, "red + blue = gold", x=Screen.Width/2 - 150, y=400, w=300)
			make_sheep(self, 140,256+20)
			make_platform(self, Screen.Width/2 - 250, 256, 500, 16)
			make_sheep(self, 140,128+20)
			make_platform(self, Screen.Width/2 - 250, 128, 500, 16)
			make_behavior(self, "Move Right", 10, 10, ConstantMovement, (vector(30.0, 0),))
			make_goal(self, 400, 256+16, 100, 100, 'Start')
			#make_goal(self, 400, 128+16, 100, 100, 'About')

		elif(n == 1):
			make_textbox(self, "Hope this doesn't bore you", x=Screen.Width/2 - 150, y=400, w=300)
			self.goals_required = 1
			make_sheep(self, 140, 256+20)
			make_platform(self, 50, 256, 250, 16)
			make_platform(self, 380, 128, 150, 16)
			make_goal(self, 400, 128+16, 50, 50, 'Goal')
			make_behavior(self, "Walk Right", 8, 8, ConstantMovement, (vector(30.0, 0),))
			make_behavior(self, "Sprint Right", 40, 8, ConstantMovement, (vector(60.0, 0),))

		#elif(n == 2):
		#	make_textbox(self, "Oh no, it moves!", x=Screen.Width/2 - 150, y=400, w=300)
		#	self.goals_required = 1
		#	make_sheep(self, 300, 256 + 20)
		#	make_platform(self, 32, 256, Screen.Width-64, 16)
		#	mover = make_platform(self, 32, 256+16, 50, 50)
		#	make_goal(self, 100, 256+16, 50, 50, 'Goal')
		#	walk = make_behavior(self, "Walk Left", 8, 8, ConstantMovement, (vector(-30.0, 0),))
		#	walk2 = make_behavior(self, "Walk Right", 8, 8, ConstantMovement, (vector(30.0, 0),))
		#	jump = make_behavior(self, "Bounce", 40, 8, BounceMovement, None)

		#	self.esys.add_component(mover, Configurable)
		#	self.esys.add_component(mover, Velocity(0,0))
		#	self.esys.add_component(mover, Selectable(mover))
		#	self.esys.get_component(walk2, Behavior).attach(mover)

		else:
			self.goals_required = 99
			make_textbox(self, "Victory!", x=Screen.Width/2 - 150, y=400, w=300)

		self.goals = 0

	def on_goal(self, entity):
		m = self.esys
		#m.add_component(entity, Finished)
		m.kill_entity(entity)

		self.goals += 1
		if self.goals >= self.goals_required:
			self.index+=1

		make_confirm(self.esys, "Level completed! Click to proceed.", lambda : self.load_level())


		#print 'level.on_goal()'

	def clear(self):
		for e in self.entities:
			self.esys.kill_entity(e)

	def add_entity(self, num):
		self.entities.append(num)
		return num

	def make_entity(self, *args):
		index = self.esys.make_entity(*args)
		return self.add_entity(index)

	def death(self, entity):
		# shouldn't be processing if not ready
		# add a gui object which will call load_level when confirmed
		make_confirm(self.esys, "Level failed. Click to retry.", lambda : self.load_level())

	def update(self, dt):
		m = self.esys

		# if there exists an entity such that the level is blocked
		# we don't update ... generally a gui thing
		if len(m.get_entities(BlocksLevel)) != 0:
			return

		def sign(a):
			if a == 0:
				return 1
			elif a < 0:
				return -1
			else:
				return 1

		for e in m.get_entities(ConstantMovement):
			move = m.get_component(e, ConstantMovement).velocity
			vel = m.get_component(e, Velocity)
			if move.x and abs(vel.x) < abs(move.x):
				vel.x = (move.x)
			if move.y and abs(vel.y) < abs(move.y):
				vel.y = (move.y)

			m.set_component(e, Velocity, vel)

		for e in m.get_entities(Velocity):
			if m.has_component(e, Draggable):
				draggable = m.get_component(e, Draggable)
				if(draggable.is_held()):
					continue

			vel  = m.get_component(e, Velocity)
			pos  = m.get_component(e, Position)
			size = m.get_component(e, Size)
			
			if m.has_component(e, SurvivalRequired):
				if(pos.y < 0):
					self.death(e)
					m.kill_entity(e)
					return
				
			# apply gravity if it makes sense
			if m.has_component(e, Gravity):
				next_vel = vel - vector(0,4)
			else:
				next_vel = vel
			m.set_component(e, Velocity, next_vel)

			# correct for any collisions
			(hit, next_pos) = collision_correction(m, e, pos + vel.scale(dt))
			
			#if(hit):
			m.set_component(e, Position, next_pos)
		

		for e in m.get_entities(WrapToWorld):
			pos = m.get_component(e, Position)
			pos.x = pos.x % Screen.Width

		finished = False
		while not finished:
			finished = self.handle_regions(m)

	def handle_regions(self, m):
		for e in m.get_entities(Position):
			if not m.has_component(e, Size):
				continue

			pos  = m.get_component(e, Position)
			size = m.get_component(e, Size)

			e_rect = rectangle(pos.x, pos.y, size.x, size.y)

			for r in m.get_entities(RegionResponse):
				if r == e:
					continue

				r_pos  = m.get_component(r, Position)
				r_size = m.get_component(r, Size)
				
				r_rect = rectangle(r_pos.x, r_pos.y, r_size.x, r_size.y)
				
				if r_rect.contains(e_rect):
					print 'fire'
					resp = m.get_component(r, RegionResponse)
					if resp.fire(e):
						m.kill_entity(r)
					return False
		return True





