from components import *
from factory import *

def collision_correction(esys, entity, next_position):
	m = esys
	next_pos = next_position
	e = entity

	hit = False

	pos = m.get_component(e, Position)
	size = m.get_component(e, Size)

	# determine if next position will be on a platform
	for p in m.get_entities(Obstacle):

		# ignore yourself in this collision detection
		if p == e:
			continue

		p_obst = m.get_component(p, Obstacle)
		p_pos = m.get_component(p, Position)
		p_size = m.get_component(p, Size)

		cur_rect = rectangle(pos.x, pos.y, size.x, size.y)
		next_rect = rectangle(next_pos.x, next_pos.y, size.x, size.y)
		obstacle = rectangle(p_pos.x, p_pos.y, p_size.x, p_size.y)

		if will_intersect(obstacle, cur_rect, next_rect):
			hit = False

			vel_mod = Velocity(1,1)
			
			if m.has_component(e, Velocity):
				vel = m.get_component(e, Velocity)
			else:
				vel = Velocity(0,0)

			dx = next_pos.x - pos.x
			dy = next_pos.y - pos.y
			
			touching = obstacle.intersects(next_rect) 

			if touching:
				# apply friction
				if m.has_component(p, Friction):
					vel_mod.x *= m.get_component(p, Friction).value

			if dx:
				if obstacle.rect_on_left(cur_rect) and (touching or not obstacle.rect_on_left(next_rect)):
					next_pos.x = p_pos.x - size.x
					vel_mod.x *= -1
				elif obstacle.rect_on_right(cur_rect) and (touching or not obstacle.rect_on_right(next_rect)):
					next_pos.x = p_pos.x + p_size.x
					vel_mod.x *= -1 
				else:
					pass
			
			if dy:
				if obstacle.rect_below(cur_rect) and (touching or not obstacle.rect_below(next_rect)):
					next_pos.y = p_pos.y - size.y
					vel_mod.y = 0
				elif obstacle.rect_above(cur_rect) and (touching or not obstacle.rect_above(next_rect)):
					next_pos.y = p_pos.y + p_size.y
					vel_mod.y = 0
					if m.has_component(e, BounceMovement):
						vel.y = 200
						vel_mod.y = 1

				else:
					pass

			# if the component has velocity, change it
			if m.has_component(e, Velocity):
				m.set_component(e, Velocity, Velocity(vel.x*vel_mod.x, vel.y*vel_mod.y))

	return (hit, next_pos)

def will_intersect(obstacle, start, end):
	#start is moving to end, so merge them
	area = start.merge(end.point).merge(end.point + end.size)
	return area.intersects(obstacle)

