from components import *
from behaviors import *
from geom import rectangle, vector

class Screen(object):
	Width = 640
	Height = 480
	@staticmethod
	def rect():
		return rectangle(0,0,Screen.Width,Screen.Height)

import pyglet
from pyglet.sprite import Sprite
from pyglet.window import mouse
from pyglet.gl import *

class DisplayText(Component):
	def __init__(self, data="<text here>"):
		self.label = pyglet.text.Label(
				text = data,
				font_name = ['Courier New', 'Courier'],
				font_size = 14.0,
				color = (0,0,0,255),
				anchor_y = 'center',
				anchor_x = 'center'
				)
	
	def __repr__(self):
		return 'DisplayText("{0}")'.format(self.label.text)
	
	def draw(self):
		self.label.draw()

class Display(object):
	def __init__(self, w, h, caption):
		self.window = pyglet.window.Window(w,h,caption)
		# so that this on_draw gets called
		self.window.push_handlers(self)
		self.esys = None
		self.width = w
		self.height = h

	def add_listener(self, handler):
		self.window.push_handlers(handler)

	def start(self):
        # set clearcolor to grey instead
		glClearColor(.9,.9,.9,1)

        # enable transparency
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		pyglet.app.run()

	def end(self):
		pyglet.app.exit()

	def set_interval(self, func, delay):
		pyglet.clock.schedule_interval(func, delay)

	def set_system(self, esys):
		self.esys = esys

	def on_draw(self):
		self.window.clear()

		m = self.esys

		if(m == None):
			return

		draw_selectable = len(m.get_entities(BlocksLevel)) != 0

		# get all depth-sorted entities
		for e in sorted(m.get_entities(Renderable),
				lambda x,y:
					m.get_component(x,Renderable).depth > m.get_component(y,Renderable).depth
					):
			
			# don't draw attached behaviors
			if m.has_component(e, Behavior):
				behavior = m.get_component(e, Behavior)
				if behavior.attached != None:
					continue

			if m.has_component(e, DisplayQuad):
				pos   = m.get_component(e, Position)
				size  = m.get_component(e, Size)
				color = m.get_component(e, FillColor)
				rect  = rectangle(pos.x,
						pos.y,
						size.x,
						size.y)

				pyglet.graphics.draw(4, pyglet.gl.GL_QUADS,
						('c4B', color.to_tuple()*4),
						('v2f', rect.to_tuple())
						)
			
			if draw_selectable:
				if m.has_component(e, Selectable):
					state = m.get_component(e, Selectable)
					if state.selected:
						pos   = m.get_component(e, Position)
						size  = m.get_component(e, Size)
						rect  = rectangle(pos.x,
								pos.y,
								size.x,
								size.y)
						
						pyglet.graphics.draw(4, pyglet.gl.GL_QUADS,
								('c4B', (255,255,255,127)*4),
								('v2f', rect.to_tuple())
								)


			if m.has_component(e, DisplayBorder):
				pos   = m.get_component(e, Position)
				size  = m.get_component(e, Size)
				style = m.get_component(e, LineStyle)
				color = style.color
				
				glLineWidth(style.width)

				rect  = rectangle(pos.x,
						pos.y,
						size.x,
						size.y)

				pyglet.graphics.draw(4, pyglet.gl.GL_LINE_LOOP,
						('c4B', color.to_tuple()*4),
						('v2f', rect.to_tuple())
						)

			if m.has_component(e, DisplayConnection):
				connection = m.get_component(e, DisplayConnection)
				a = connection.e_from
				b = connection.e_to

				pos_a  = m.get_component(a, Position)
				size_a = m.get_component(a, Size)
				pos_b  = m.get_component(b, Position)
				size_b = m.get_component(b, Size)

				vertex_list = (pos_a.x + size_a.x/2, pos_a.y + size_a.y/2,
						       pos_b.x, pos_b.y + size_b.y)

				style = m.get_component(e, LineStyle)
				glLineWidth(style.width)

				pyglet.graphics.draw(2, pyglet.gl.GL_LINES,
						('c4B', style.color.to_tuple()*2),
						('v2f', vertex_list)
						)

			# draw all things with text last
			if m.has_component(e, DisplayText):
				loc = m.get_component(e, Position)
				size = m.get_component(e, Size)
				label = m.get_component(e, DisplayText).label
				label.x = loc.x+size.x/2
				label.y = loc.y+size.y/2
				label.color = m.get_component(e, FontColor).to_tuple()
				m.get_component(e, DisplayText).draw()


